function [n,msg1,msg2] = firchk(n,a,exception)
%FIRCHK   Check if specified filter order is valid.
%   FIRCHK(N,A) checks if the specified order N is valid given
%   the desired magnitude response vector A.  Type 2 linear
%   phase FIR filters (symmetric, odd order) must have a
%   desired magnitude response vector that ends in zero.  This
%   is because type 2 filters necessarily have a zero at w = pi.
%
%   If the order is not valid, a warning is given and the order
%   of the filter is incremented by one.
%
%   If A is a scalar (as when called from fircls1), A = 0 is
%   interpreted as lowpass and A = 1 is interpreted as highpass.
%
%   FIRCHK(N,A,EXCEPTION) will not warn or increase the order
%   if EXCEPTION = 1.  Examples of EXCEPTIONS are type 4 filters
%   (such as differentiators or hilbert transformers) or non-linear
%   phase filters (such as minimum and maximum phase filters).

%   Author : R. Losada
%   Copyright 1988-2002 The MathWorks, Inc.
%   $Revision: 1.7 $  $Date: 2002/03/28 17:29:42 $

nargchk(2,3,nargin);

if nargin == 2,
    exception = 0;
end

msg1 = '';
msg2 = '';
oddord = 0; % Flag, initially we assume even order

if isempty(n) | ~isnumeric(n) | ~isreal(n) | n~=round(n) | n<=0,
    msg1 = 'Filter order must be a real, positive integer.';
    return
end

if rem(n,2),
    oddord = 1; % Overwrite flag
end
 
if (a(end) ~= 0) & oddord & ~exception,
    str = ['Odd order symmetric FIR filters must have a gain of zero \r'...
     'at the Nyquist frequency. The order is being increased by one.'];
    msg2 = sprintf(str);
    n = n+1;
end
    
